import random;
from math import sin,cos,sqrt;
from geometry2d import pi,Points,Vectors,Lines,Segments;

import numpy as np;
import scipy.stats as spst;

# Dielengerade, hier einfach die x-Achse
floorLine=Lines.toLineWithDir(Points(0,0),Vectors(1,0));

random.seed();

# Wurf der Nadel
def needlePitch() -> bool:
    yNdM=random.uniform(-0.5,0.5);
    # Zufällige Lage der geworfenen Nadel auf der y-Achse
    # im Intervall (-½,½)

    PtNdM=Points(0.0,yNdM);
    # Punkt der Nadelmitte auf der Diele

    phiNd=random.uniform(-pi/2,pi/2);
    # Zufällige Orientierung der Nadel auf der Diele
    # im Intervall (-π/2,π/2)

    DvNd=Vectors(sin(phiNd),cos(phiNd));
    # Richtungsvektor der Nadel auf der Diele

    needle=Segments.toSegmentWithDir(PtNdM,DvNd,1);
    # Erzeugung des Segments (Strecke) aus der Nadelmitte,
    # dem Richtungsvektor und der Nadellänge 1

    return needle.isIntersectionOf_withLine(floorLine);
    # Test, ob die Strecke die Gerade schneidet

# end def needlePitch

# ------------------------------------------------------------------------------#
# ------------------------------------------------------------------------------#

if __name__=='__main__':
    print('\n# Beginning Pi-mit-Dielenbretter-nruns ...\n');

    # ------------------------------------------------------------------------------#
    # ------------------------------------------------------------------------------#

    Runs=10;
    # Anzahl der Durchläufe (Runs)
    Pitches=1600000; # Mein Standardwert 2000000
    # Anzahl der Würfe
    RunResult=0.0;
    RunResults: list[float]=[];

    # ------------------------------------------------------------------------------#

    for irun in range(1,Runs+1,1):
        print("Doing run : ",irun);
        crossed=0;
        # Nadel überkreuzt die Dielengerade
        for ipitch in range(1,Pitches+1,1):
            if needlePitch(): crossed+=1;
        # end for
        RunResults.append(crossed/Pitches);
        # Wahrscheinlichkeiten für die Überkreuzung
    # end for

    N=len(RunResults);
    MeanRunResult=np.mean(RunResults);
    sigma=np.std(RunResults,ddof=1);
    p=0.99; # 0.95
    d=spst.t(N-1).interval(p)[1];
    d=d*sigma/sqrt(N);

    # ------------------------------------------------------------------------------#
    print ("# -------------------------------------#");
    print("Nb of Runs    : ",Runs);
    print("Nb of Pitches : ",Pitches);
    print("Propability % : ",MeanRunResult*100," %");
    print("|Result-2/pi| : ",abs(MeanRunResult-2.0/pi));
    print("sigma         : ",sigma);
    print("p             : ",p)
    print("Intervall     : ",[MeanRunResult-d,MeanRunResult+d]);

    # ------------------------------------------------------------------------------#
    # ------------------------------------------------------------------------------#
    print('\n# Finished Pi-mit-Dielenbretter-nruns.\n');
# end if main
